#include <stdio.h>
#include <stdlib.h>
#include <CUnit/Basic.h>
#include <stdbool.h>
#include "Chaine.h"

int init_suite_success(void) { 
  return 0; 
}

int clean_suite_success(void) { 
  return 0; 
}

/* Chaines constantes */

CH_Chaine chaine_vide() {
  CH_Chaine res;
  res.caracteres = LCDC_listeChainee();
  res.longueur = 0;
  return res;
}


LCDC_ListeChainee liste_abc() {
  LCDC_ListeChainee res = LCDC_listeChainee();
  LCDC_ajouter(&res,'c');
  LCDC_ajouter(&res,'b');
  LCDC_ajouter(&res,'a');
  return res;
}

CH_Chaine chaine_abc() {
  CH_Chaine res;
  res.caracteres = liste_abc();
  res.longueur = 3;
  return res;
}

LCDC_ListeChainee liste_def() {
  LCDC_ListeChainee res = LCDC_listeChainee();
  LCDC_ajouter(&res,'f');
  LCDC_ajouter(&res,'e');
  LCDC_ajouter(&res,'d');
  return res;
}

CH_Chaine chaine_def() {
  CH_Chaine res;
  res.caracteres = liste_def();
  res.longueur = 3;
  return res;
}

LCDC_ListeChainee liste_abcdef() {
  LCDC_ListeChainee res = liste_def();
  LCDC_ajouter(&res,'c');
  LCDC_ajouter(&res,'b');
  LCDC_ajouter(&res,'a');  
  return res;
}

CH_Chaine chaine_abcdef() {
  CH_Chaine res;
  res.caracteres = liste_abcdef();
  res.longueur = 6;
  return res;
}

LCDC_ListeChainee liste_cba() {
  LCDC_ListeChainee res = LCDC_listeChainee();
  LCDC_ajouter(&res,'a');
  LCDC_ajouter(&res,'b');
  LCDC_ajouter(&res,'c');
  return res;
}

CH_Chaine chaine_cba() {
  CH_Chaine res;
  res.caracteres = liste_cba();
  res.longueur = 3;
  return res;
}

/* Test unitaires */

void test_CreerChaineVide() {
  CH_Chaine ch1 = CH_chaine("");
  char *s = CH_enCharEtoile(ch1);
  CU_ASSERT_STRING_EQUAL(s,"");
  CH_liberer(&ch1);
  free(s);
}

void test_CreerChaineNonVide() {
  CH_Chaine ch1 = CH_chaine("abc");
  char *s = CH_enCharEtoile(ch1);
  CU_ASSERT_STRING_EQUAL(s,"abc");
  CH_liberer(&ch1);
  free(s);
}


void test_LongueurChaineVide() {
  CH_Chaine ch = chaine_vide();
  CU_ASSERT_TRUE(CH_longueur(ch)==0);
  CH_liberer(&ch);
}

void test_LongueurChaineNonVide() {
  CH_Chaine ch = chaine_abcdef();
  CU_ASSERT_TRUE(CH_longueur(ch)==6);
  CH_liberer(&ch);
}

void test_CopierChaineVide() {
  CH_Chaine ch1 = chaine_vide();
  CH_Chaine ch2 = CH_copier(ch1);
  char *s = CH_enCharEtoile(ch2);
  CU_ASSERT_STRING_EQUAL(s,"");
  CH_liberer(&ch1);
  CH_liberer(&ch2);
  free(s);
}

void test_CopierChaineNonVide() {
  CH_Chaine ch1 = chaine_abc();
  CH_Chaine ch2 = CH_copier(ch1);
  char *s = CH_enCharEtoile(ch2);
  CU_ASSERT_STRING_EQUAL(s,"abc");
  CH_liberer(&ch1);
  CH_liberer(&ch2);
  free(s);
}

void test_ConcatenerPDeuxChainesVides() {
  CH_Chaine ch1 = chaine_vide();
  CH_Chaine ch2 = chaine_vide();
  CH_concatenerP(&ch1,ch2);
  char *s = CH_enCharEtoile(ch1);
  CU_ASSERT_STRING_EQUAL(s,"");
  CH_liberer(&ch1);
  CH_liberer(&ch2);
  free(s);
}

void test_ConcatenerPChaineVideEnPremier() {
  CH_Chaine ch1 = chaine_vide();
  CH_Chaine ch2 = chaine_def();
  CH_concatenerP(&ch1,ch2);
  char *s = CH_enCharEtoile(ch1);
  CU_ASSERT_STRING_EQUAL(s,"def");
  CH_liberer(&ch1);
  CH_liberer(&ch2);
  free(s);
}

void test_ConcatenerPChaineVideEnDeuxieme() {
  CH_Chaine ch1 = chaine_abc();
  CH_Chaine ch2 = chaine_vide();
  CH_concatenerP(&ch1,ch2);
  char *s = CH_enCharEtoile(ch1);
  CU_ASSERT_STRING_EQUAL(s,"abc");
  CH_liberer(&ch1);
  CH_liberer(&ch2);
  free(s);
}

void test_ConcatenerPGeneral() {
  CH_Chaine ch1 = chaine_abc();
  CH_Chaine ch2 = chaine_def();
  CH_concatenerP(&ch1,ch2);
  char *s = CH_enCharEtoile(ch1);
  CU_ASSERT_STRING_EQUAL(s,"abcdef");
  CH_liberer(&ch1);
  CH_liberer(&ch2);
  free(s);
}

void test_ConcatenerPLongueur() {
  CH_Chaine ch1 = chaine_abc();
  CH_Chaine ch2 = chaine_def();
  CH_concatenerP(&ch1,ch2);
  CU_ASSERT_TRUE(CH_longueur(ch1)==6);
  CH_liberer(&ch1);
  CH_liberer(&ch2);
}

void test_ConcatenerFDeuxChainesVides() {
  CH_Chaine ch1 = chaine_vide();
  CH_Chaine ch2 = chaine_vide();
  CH_Chaine ch3 = CH_concatenerF(ch1,ch2);
  char *s = CH_enCharEtoile(ch3);
  CU_ASSERT_STRING_EQUAL(s,"");
  CH_liberer(&ch1);
  CH_liberer(&ch2);
  CH_liberer(&ch3); 
  free(s);
}

void test_ConcatenerFChaineVideEnPremier() {
  CH_Chaine ch1 = chaine_vide();
  CH_Chaine ch2 = chaine_def();
  CH_Chaine ch3 = CH_concatenerF(ch1,ch2);
  char *s = CH_enCharEtoile(ch3);
  CU_ASSERT_STRING_EQUAL(s,"def");
  CH_liberer(&ch1);
  CH_liberer(&ch2);
  CH_liberer(&ch3); 
  free(s);
}

void test_ConcatenerFChaineVideEnDeuxieme() {
  CH_Chaine ch1 = chaine_abc();
  CH_Chaine ch2 = chaine_vide();
  CH_Chaine ch3 = CH_concatenerF(ch1,ch2);
  char *s = CH_enCharEtoile(ch3);
  CU_ASSERT_STRING_EQUAL(s,"abc");
  CH_liberer(&ch1);
  CH_liberer(&ch2);
  CH_liberer(&ch3); 
  free(s);
}

void test_ConcatenerFGeneral() {
  CH_Chaine ch1 = chaine_abc();
  CH_Chaine ch2 = chaine_def();
  CH_Chaine ch3 = CH_concatenerF(ch1,ch2);
  char *s = CH_enCharEtoile(ch3);
  CU_ASSERT_STRING_EQUAL(s,"abcdef");
  CH_liberer(&ch1);
  CH_liberer(&ch2);
  CH_liberer(&ch3); 
  free(s);
}

void test_ConcatenerFLongueur() {
  CH_Chaine ch1 = chaine_abc();
  CH_Chaine ch2 = chaine_def();
  CH_Chaine ch3 = CH_concatenerF(ch1,ch2);
  CU_ASSERT_TRUE(CH_longueur(ch3)==6);
  CH_liberer(&ch1);
  CH_liberer(&ch2);
  CH_liberer(&ch3); 
}

void test_InverserChaineVide() {
  CH_Chaine ch = chaine_vide();
  CH_inverser(&ch);
  char *s = CH_enCharEtoile(ch);
  CU_ASSERT_STRING_EQUAL(s,"");
  CH_liberer(&ch); 
}

void test_InverserGeneral() {
  CH_Chaine ch = chaine_abc();
  CH_inverser(&ch);
  char *s = CH_enCharEtoile(ch);
  CU_ASSERT_STRING_EQUAL(s,"cba");
  CH_liberer(&ch); 
}

int main(int argc, char** argv){
  CU_pSuite pSuite = NULL;

  /* initialisation du registre de tests */
  if (CUE_SUCCESS != CU_initialize_registry())
    return CU_get_error();

  /* ajout d'une suite de test */
  pSuite = CU_add_suite("Test chaine boite noire", init_suite_success, clean_suite_success);
  if (NULL == pSuite) {
    CU_cleanup_registry();
    return CU_get_error();
  }
  /* Ajout des tests à la suite de tests boite blanches */
  if ((NULL == CU_add_test(pSuite, "Creation d'une chaine vide", test_CreerChaineVide))
      || (NULL == CU_add_test(pSuite, "Creation d'une chaine non vide", test_CreerChaineNonVide))
      || (NULL == CU_add_test(pSuite, "Longueur d'une chaine vide", test_LongueurChaineVide))
      || (NULL == CU_add_test(pSuite, "Longueur d'une chaine non vide", test_LongueurChaineNonVide))
      || (NULL == CU_add_test(pSuite, "Copier chaine vide", test_CopierChaineVide))
      || (NULL == CU_add_test(pSuite, "Copier chaine non vide", test_CopierChaineNonVide))
      || (NULL == CU_add_test(pSuite, "ConcatenerP deux chaines vides", test_ConcatenerPDeuxChainesVides))
      || (NULL == CU_add_test(pSuite, "ConcatenerP chaine vide en premier", test_ConcatenerPChaineVideEnPremier))
      || (NULL == CU_add_test(pSuite, "ConcatenerP chaine vide en deuxieme", test_ConcatenerPChaineVideEnDeuxieme))
      || (NULL == CU_add_test(pSuite, "ConcatenerP cas general", test_ConcatenerPGeneral))
      || (NULL == CU_add_test(pSuite, "ConcatenerP longueur cas general", test_ConcatenerPLongueur))
      || (NULL == CU_add_test(pSuite, "ConcatenerF deux chaines vides", test_ConcatenerFDeuxChainesVides))
      || (NULL == CU_add_test(pSuite, "ConcatenerF chaine vide en premier", test_ConcatenerFChaineVideEnPremier))
      || (NULL == CU_add_test(pSuite, "ConcatenerF chaine vide en deuxieme", test_ConcatenerFChaineVideEnDeuxieme))
      || (NULL == CU_add_test(pSuite, "ConcatenerF cas general", test_ConcatenerFGeneral))
      || (NULL == CU_add_test(pSuite, "ConcatenerF longueur cas general", test_ConcatenerFLongueur))
      || (NULL == CU_add_test(pSuite, "Inverser chaine vide", test_InverserChaineVide))
      || (NULL == CU_add_test(pSuite, "Inverser cas general", test_InverserGeneral))  
      )
    {
      CU_cleanup_registry();
      return CU_get_error();
    }

  /* Lancement des tests */
  CU_basic_set_mode(CU_BRM_VERBOSE);
  CU_basic_run_tests();
  printf("\n");
  CU_basic_show_failures(CU_get_failure_list());
  printf("\n\n");

  /* Nettoyage du registre */
  CU_cleanup_registry();
  return CU_get_error();
}
